#ifndef _HATPPLAN_H
#define _HATPPLAN_H

#include "hatpNode.hh"
#include "hatpStreams.hh"
#include "hatpTree.hh"
#include "hatpLexer.hh"
#include "hatpTools.hh"

/**
 * Static method for testing if an input seems to be a plan structure
 * @param content A string representing a plan
 * @return true if the input seems to be a plan structure
 */
static bool testInputValidity(const std::string& content) {
	return (boost::regex_search(content, regexValidity));
}


/**
 * Main Plan Class.
 * hatpPlan is the structure representing and allowing to manipulate a plan produced by HATP
 */
class hatpPlan {

	private:
		bool validPlan;
		unsigned int planID;
		std::string planReport;
		std::map<unsigned int, hatpNode*> nodes;
		hatpStreams* streams;
		hatpTree* tree;
		
	protected:

		/**
		 * Used by the constructor to create the list of nodes.
		 * @see hatpPlan()
		 * @param content A string representing an action
		 */
		void constructNodeList(std::string const& content);

		/**
		 * Used by the constructor to create the list of links of each node.
		 * @see hatpPlan()
		 * @param content A string representing a link
		 * @param type A string indicating the type (causal or hierarchical) of this link
		 */
		void constructLinkList(std::string const& content, std::string const& type);
	
	public:

		/**
		 * Constructor of the Plan structure.
		 * Construct a plan representation starting from a message received by HATP that is parsed.
		 * @param plan A string representing the plan received from HATP
		 */
		hatpPlan(std::string const& plan);
		
		/**
		 * Empty destructor.
		 */
		~hatpPlan();

		/**
		 * Get the plan validity.
		 * @return true if the plan is valid, ie. the input is a plan structure
		*/
		bool isPlanValid() const;

		/**
		 * Get the report status OK.
		 * @return true if the report is OK
		*/
		bool isReportOK() const;

		/**
		 * Get the report status NOPLAN.
		 * @return true if the report is NOPLAN
		*/
		bool isReportNoPlan() const;

		/**
		 * Get the report status ALREADY ACHIEVED.
		 * @return true if the report is ALREADY ACHIEVED
		*/
		bool isReportAlreadyAchieved() const;

		/**
		 * Get the plan ID.
		 * @return The ID of the current plan
		*/
		unsigned int getID() const;

		/**
		 * Get the plan report.
		 * @return The report of the current plan
		 */
		std::string getReport() const;

		/**
		 * Get the list of nodes.
		 * @return A vector containing pointers to each node
		 */
		std::vector<hatpNode*> getNodes() const;

		/**
		 * Get the list of nodes of type "action".
		 * @return A vector containing pointers to nodes of type action
		 */
		std::vector<hatpNode*> getActionNodes() const;

		/**
		 * Get a specific node.
		 * @param n The ID of the node
		 * @return a pointer to the node
		 */
		hatpNode* getNode(unsigned int n) const;

		/**
		 * Get the plan as streams.
		 * @return A pointer towards the streams structure
		 */
		hatpStreams* getStreams() const;

		/**
		 * Get the plan as a tree.
		 * @return A pointer towards the tree structure
		 */
		hatpTree* getTree() const;

		/**
		 * Get the list of agents involved in the current plan.
		 * @return A vector containing the agent's name
		 */
		std::vector<std::string> getAgentsList() const;

		/**
		 * Get the plan under a string form.
		 * @return a string representing the plan under a text form
		 */
		std::string toString() const;

};

#endif

